<?php

namespace Financeiro\Logic\ContasReceber\BoletosDescontados;

use Financeiro\Entities\Baixarecebida;
use Financeiro\Entities\Boletreceber;
use Financeiro\Entities\Contareceber;
use Geral\Entities\Lanccont;
use Geral\Util\Ambiente;
use \Geral\Util\DateUtil;
use \Setforms2\Shared\Registry;
use \Geral\Util\NovoCodigo;
use \Financeiro\Entities\Movcredbanc;
use \Financeiro\Entities\Movbancario;
use Financeiro\Logic\AtualizaFluxodeCaixa\FluxodeCaixa;

class BoletosDescontados
{
    public static function excluirMovcredbanc($cdreducont, $nrmovicred, $nrorg)
    {
        try {
            $movcredbanc = Movcredbanc::findOneByCdreducontAndNrmovicredAndNrorg($cdreducont, $nrmovicred, $nrorg);
            $movcredbanc->delete();
            $movcredbanc->persist();
            Registry::getEntityManager()->flush();
        } catch(\Exception $e)
        {
            throw $e;
        }
    }

    public static function excluirMovbancario($cdreducont, $nrmovbanc, $nrorg)
    {
        try {
            $movbancario = Movbancario::findOneByCdreducontAndNrmovbancAndNrorg($cdreducont, $nrmovbanc, $nrorg);
            $movbancario->delete();
            $movbancario->persist();
            Registry::getEntityManager()->flush();
        } catch(\Exception $e)
        {
            throw $e;
        }
    }

    public static function excluirLancamentoContabil($cdempresa, $nrlanccont, $nrorg){
        try {
            /** @var Lanccont $lanccont */
            $lanccont = Lanccont::findByNrorgAndCdempresaAndNrlanccont($nrorg, $cdempresa, $nrlanccont);
            foreach($lanccont as $partida) {
                if ($lanccont instanceof Lanccont) {
                    $lanccont->delete();
                    $lanccont->persist();
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function inserirMovimentoBancario($vrmovbanc, $cdreducont, $nrdocmbanc, $dshismbanc, $nrorg, $dataatual)
    {
        try{
            $movbancario = new \Financeiro\Entities\Movbancario();
            $movbancario->setCdreducont($cdreducont);
            $movbancario->setDtinclusao($dataatual);
            $movbancario->setDtultatu($dataatual);
            $movbancario->setDtmovbanc($dataatual);
            $movbancario->setDshismbanc($dshismbanc);
            $movbancario->setNrdocmbanc($nrdocmbanc);
            $movbancario->setIdmovbanc('C');
            $movbancario->setVrmovbanc($vrmovbanc);
            $movbancario->setIdorigmbanc('C');
            $movbancario->setIdstconext('0');
            $movbancario->setNrorg($nrorg);
            $nrmovbanc = NovoCodigo::geraCodigo($nrorg, 'MOVBANCARIO'.$cdreducont);
            $movbancario->setNrmovbanc($nrmovbanc);
            $movbancario->persist();
            Registry::getEntityManager()->flush();

            return $nrmovbanc;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function inserirCreditoBancario(
        $cdreducont, $cdclassfina, $cdempresa, $cdfilial, $cdtpcredbanc, $dsmovicred, $vrmovicred, $nrlctbmcred,
        $cdfililancre, $nrorg, $dataatual, $nrmovbanc
    )
    {
        try {
            $movcredbanc = new \Financeiro\Entities\Movcredbanc();
            $movcredbanc->setCdreducont($cdreducont);
            $nrmovicred = NovoCodigo::geraCodigo($nrorg, 'MOVCREDBANC'.$cdreducont);
            $movcredbanc->setNrmovicred($nrmovicred);
            $movcredbanc->setCdclassfina($cdclassfina);
            $movcredbanc->setCdempresa($cdempresa);
            $movcredbanc->setCdfilial($cdfilial);
            $movcredbanc->setDtmovfluxo($dataatual);
            $movcredbanc->setIdmovfluxo('R');
            $movcredbanc->setNrmovbanc($nrmovbanc);
            $movcredbanc->setCdtpcredbanc($cdtpcredbanc);
            $movcredbanc->setDtmovicred($dataatual);
            $movcredbanc->setDsmovicred($dsmovicred);
            $movcredbanc->setVrmovicred($vrmovicred);
            $movcredbanc->setCdectbmcred($cdempresa);
            $movcredbanc->setNrlctbmcred($nrlctbmcred);
            $movcredbanc->setCdfililancre($cdfililancre);
            $movcredbanc->setIdativo('S');
            $movcredbanc->setNrorg($nrorg);
            $movcredbanc->setDtinclusao($dataatual);
            $movcredbanc->setDtultatu($dataatual);
            $movcredbanc->setIdlancomcontrec('N');
            $movcredbanc->setNrorginclusao($nrorg);
            $movcredbanc->setCdoperultatu($nrorg);
            $movcredbanc->setNrorgultatu($nrorg);

            $movcredbanc->persist();
            Registry::getEntityManager()->flush();

            return $nrmovicred;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function updateContaReceber($boletos, $nrorg, $valor, $status, $idmovfluxo)
    {
        try {
            for ($i = 0; $i < count($boletos); $i++)
            {
                /** @var $contasreceber Contareceber */
                $contasreceber = Contareceber::findByCdfilibolrecAndNrlanbolrecAndNrorg($boletos[$i]['CDFILIAL'], $boletos[$i]['NRLANBOLREC'], $nrorg);
                /** @var $contareceber Contareceber */
                foreach($contasreceber as $contareceber) {
                    FluxodeCaixa::atualizaFluxo(
                        $contareceber->getCdempresa(), $contareceber->getCdfilial(), $contareceber->getCdclassfina(), $contareceber->getDtmovfluxo(), $valor, $idmovfluxo
                    );
                    $contareceber->setIdstatrec($status);
                    $contareceber->persist();
                }
            }

            Registry::getEntityManager()->flush();

        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function updateBoletReceber($boletos, $cdreducont, $nrmovicred, $nrorg, $cartcobrec, $status)
    {
        try {
            for ($i = 0; $i < count($boletos); $i++)
            {
                /** @var $boletreceber Boletreceber */
                $boletreceber = Boletreceber::findOneByCdfilialAndNrlanbolrecAndNrorg($boletos[$i]['CDFILIAL'], $boletos[$i]['NRLANBOLREC'], $nrorg);
                $boletreceber->setCdreducont($cdreducont);
                $boletreceber->setNrmovicred($nrmovicred);
                $boletreceber->setIdcartcobrec($cartcobrec);
                $boletreceber->setIdstatbrec($status);
                $boletreceber->persist();
            }

            Registry::getEntityManager()->flush();

        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function updateBoletoDescontado($nrlanbolrec, $cdfilial, $nrorg){
        try {
            /** @var $boleto Boletreceber */
            $boleto = Boletreceber::findOneByNrlanbolrecAndCdfilialAndNrorg($nrlanbolrec, $cdfilial, $nrorg);
            $boleto->setIdcartcobrec(NULL);
            $boleto->setCdfilibairec(NULL);
            $boleto->setNrbaixarec(NULL);
            $boleto->setNrmovicred(NULL);
            if(!$boleto->getCdbarrarec()){
                $idstatbrec = 'A';
            }else {
                $idstatbrec = 'E';
            }
            $boleto->setIdstatbrec($idstatbrec);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function insereBaixaPlanejada($cdempresa, $cdreducont, $dtbaixarec, $vrbaixarec){
        try {
            $cdfilial = Ambiente::getCdFilial();

            $baixarec = new Baixarecebida();
            $baixarec->setCdfilial($cdfilial);
            $nrbaixarec = NovoCodigo::geraCodigo(Ambiente::getNrorg(), 'BAIXARECEBIDA'.$cdfilial);
            $baixarec->setNrbaixarec($nrbaixarec);
            $baixarec->setCdempresa($cdempresa);
            $baixarec->setCdreducont($cdreducont);
            $baixarec->setDtbaixarec($dtbaixarec);
            $baixarec->setVrbaixarec($vrbaixarec);
            $baixarec->setIdstatusbrec("P");
            $baixarec->setDsbaixarec("Baixa temporária de boleto descontado.");
            $baixarec->setNrorg(Ambiente::getNrorg());
            $baixarec->setDtinclusao($dtbaixarec);
            $baixarec->setIdativo("S");
            $baixarec->persist();

            Registry::getEntityManager()->flush();
            return array("NRBAIXAREC" => $nrbaixarec, "CDFILIBAIREC" => $cdfilial);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function atualizaBoletoBaixa(
        $boletos, $cdfilibairec = null, $nrbaixarec = null, $idstatbrec = null, $cdtpbaixarec = null
    ){
        try {
            foreach ($boletos as $boleto) {
                /** @var $boletoEntity Boletreceber */
                $boletoEntity = Boletreceber::findOneByNrlanbolrecAndCdfilialAndNrorg(
                    $boleto["NRLANBOLREC"], $boleto["CDFILIAL"], $boleto["NRORG"]
                );
                if($cdfilibairec){
                    $boletoEntity->setCdfilibairec($cdfilibairec);
                }
                if($nrbaixarec){
                    $boletoEntity->setNrbaixarec($nrbaixarec);
                }
                if($idstatbrec){
                    $boletoEntity->setIdstatbrec($idstatbrec);
                }
                if($cdtpbaixarec){
                    $boletoEntity->setCdtpbaixarec($cdtpbaixarec);
                }
                $boletoEntity->persist();
            }

            Registry::getEntityManager()->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function updateParaBaixaReal($cdfilial, $nrbaixarec, $dtbaixarec, $nrlctbbrec, $cdtpbaixarec, $vrbaixarec){
        try {
            $baixa = Baixarecebida::findOneByCdfilialAndNrbaixarecAndNrorg($cdfilial, $nrbaixarec, Ambiente::getNrorg());
            if ($baixa instanceof Baixarecebida) {
                $baixa->setIdstatusbrec("E");
                $baixa->setCdoperador(Ambiente::getCdOperador());
                $baixa->setDtlancbaixar(DateUtil::getDataAtual());
                $baixa->setDtbaixarec(self::formataData($dtbaixarec));
                $baixa->setNrlctbbrec($nrlctbbrec);
                $baixa->setCdtpbaixarec($cdtpbaixarec);
                $baixa->setVrbaixarec($vrbaixarec);
                $baixa->persist();

                Registry::getEntityManager()->flush();
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function atualizaContaBaixada($cdfilibolrec, $nrlanbolrec){
        try {
            $contas = Contareceber::findByNrorgAndCdfilibolrecAndNrlanbolrec(Ambiente::getNrorg(), $cdfilibolrec, $nrlanbolrec);
            /** @var $conta Contareceber */
            foreach ($contas as $conta) {
                $conta->setIdstatrec("B");
                $conta->persist();
            }

            Registry::getEntityManager()->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function geraLancamentoContabil(
        $cdempresa, $nrlanccont, $nrseqlanc, $cdcontctbl, $cdhistpadr, $dtlanccont, $cddocucont, $idtipolanc, $dshistlanc,
        $vrlancament, $idlanccome, $cdcontcome, $idlancsaldo, $idlancinte, $idlancmani, $cdoperins, $dtultatulan
    ){
        try {
            $lanccont = new Lanccont();
            $lanccont->setCdempresa($cdempresa);
            $dataatual = DateUtil::getDataAtual();
            $lanccont->setNrlanccont($nrlanccont);
            $lanccont->setNrseqlanc($nrseqlanc);
            $lanccont->setNrorg(Ambiente::getNrorg());
            $lanccont->setCdcontctbl($cdcontctbl);
            $lanccont->setCdhistpadr($cdhistpadr);
            $lanccont->setDtlanccont($dtlanccont);
            $lanccont->setCddocucont($cddocucont);
            $lanccont->setIdtipolanc($idtipolanc);
            $lanccont->setDshistlanc($dshistlanc);
            $lanccont->setVrlancament($vrlancament);
            $lanccont->setIdlanccome($idlanccome);
            $lanccont->setCdcontcome($cdcontcome);
            $lanccont->setIdlancsaldo($idlancsaldo);
            $lanccont->setIdlancinte($idlancinte);
            $lanccont->setIdlancmani($idlancmani);
            $lanccont->setCdoperins($cdoperins);
            $lanccont->setDtultatulan($dtultatulan);
            $lanccont->setDtinclusao($dataatual);
            $lanccont->setIdativo("S");
            $lanccont->persist();

            Registry::getEntityManager()->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function cancelaRecebimento($boletos){
        try {
            /** @var Boletreceber $boleto */
            foreach ($boletos as $boleto) {
                $boletoEntity = Boletreceber::findOneByNrlanbolrecAndCdfilialAndNrorg(
                    $boleto["NRLANBOLREC"], $boleto["CDFILIAL"], $boleto["NRORG"]
                );
                $boletoEntity->setCdfilibairec(NULL);
                $boletoEntity->setNrbaixarec(NULL);
                $boletoEntity->persist();
            }

            Registry::getEntityManager()->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function updateOrDeleteBaixa($boletos){
        try {
            foreach ($boletos as $boleto) {
                $baixaRepo = Baixarecebida::getRepository()->verificaBoletosBaixa(
                    $boleto["CDFILIBAIREC"], $boleto["NRBAIXAREC"], $boleto["NRORG"]
                );

                if ((bool)$baixaRepo == false) {
                    $baixa = Baixarecebida::findOneByCdfilialAndNrbaixarecAndNrorg(
                        $boleto["CDFILIBAIREC"], $boleto["NRBAIXAREC"], $boleto["NRORG"]
                    );

                    if ($baixa instanceof Baixarecebida) {
                        $baixa->delete();
                    }
                }
            }

            Registry::getEntityManager()->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function formataData($data)
    {
        if (strlen($data) == 19)
            $data = DateUtil::truncateString($data);
        $data = DateUtil::getDatadeString($data);
        $data = DateUtil::truncateData($data);

        return $data;
    }
}





